
; 
; 
; 
@PS_open.pro
@xdr_open.pro


PRO xinteranimate_error
	tmp = dialog_message(['Error in XINTERANIMATE:','', $
	'Only one animation at a time is allowed.'],/error)
END
;------------------------------------------------------------
PRO volviewer_readCloud,vol
; read the election cloud data
	get_lun,lun
	vol =bytarr(64,64,64)
	openr,lun,/xdr,demo_filepath('hipiph.dat', $
		subdir=['examples','demo','demodata'])
	readu,lun,vol
	close,lun
END
;------------------------------------------------------------
PRO volviewer_readvolhistory,vol_array,write=write,FILE=FILE
;
; Read volume history data from 'vol_history.xdr' file 
; which contains the following data
;
;   dim[4]  - contains [width,height,depth,nsteps]
;   ctab[256,3]   - color table
;   vol[width,height,depth]    - nstep of volume data
; KEYWORD:
;   WRITE  - if specified, then the 
;            vol_array[width,height,depth,nsteps] will be saved
;            in temporary vol_rawhistory.xdr file
;

	fname = 'vol_history.xdr'
	if keyword_set(FILE) then fname = FILE
	xdr_open,unit,fname
	xdr_read,unit,dim
	xdr_read,unit,ctab
	vol_array = make_array(dim(0),dim(1),dim(2),dim(3))
	for i=0,dim(3)-1 do begin
	xdr_read,unit,vol
	vol_array(*,*,*,i) = vol
	end
	xdr_close,unit

	if keyword_set(write) then begin
	xdr_open,unit,'vol_rawhistory.xdr',/write
	xdr_write,unit,vol_array
	xdr_close,unit
	end
END

;------------------------------------------------------------
PRO volviewer_polyshade,level,clouds,animate=animate,high=high
; This routine animate the polyshade of an volume array
; INPUT: 
;  Level  - equal surface contour level, default 0.1
;  Clouds[width,height,depth]   - 3D data array default to clouds data
; KEYWORD:
;  HIGH   - if set contour surface enclose high value , otherwise 
;           enclose low value 
;  ANIMATE - animate the polyshade surface
;

if n_elements(level) eq 0 then level = 0.1

if n_elements(clouds) eq 0 then begin
	RESTORE, FILEPATH('clouds3d.dat', SUBDIR=['examples','data'])
end
	s = SIZE(clouds)

	; Create the contour surface polygons (v and p)
	; at density 0.1, from clouds. Show the low side:
	if keyword_set(high) then $
	SHADE_VOLUME, clouds, level, v, p else $
	SHADE_VOLUME, clouds, level, v, p, /LOW

; Use SCALE3 to establish the three-dimensional
; transformation matrix. Rotate 45 degrees about the z-axis:
	SCALE3, XRANGE=[0,S[1]], YRANGE=[0,S[2]], $
	   ZRANGE=[0,S[3]], AX=0, AZ=45

; Render and display the polygons:
	im = POLYSHADE(v, p, /T3D)
help,im
	TV, im

	if keyword_set(animate) then begin
	sz = size(im)

	; Define number of views:
	nframes = 20 
	xinteranimate,set=[sz(1),sz(2),nframes],/showload

	FOR i = 0, nframes - 1 DO BEGIN & $
	   ; Translate the center of the (0, 1) unit cube
	   ; to (0,0) and rotate about the x-axis:
	   T3D, TR=[-.5, -.5, -.5], ROT=[0, 360./NFRAMES, 0] & $
	   ; Translate the center back to (0.5, 0.5, 0.5):
	   T3D, TR = [.5, .5, .5] & $
	   ; Show the surface:
	   im = POLYSHADE(v, p, /T3D) & $
	;   TV, im
	xinteranimate,frame=i,image=im

	ENDFOR

        xinteranimate,2
		
	end
END

;------------------------------------------------------------
PRO volviewer_project,vol,img,Rotate=Rotate,depth_q=depth_q,Rxyz=Rxyz, $
Opq=opq, contrast=contrast,width=width,height=height,highlight=highlight, $
transform=transform,showload=showload
; Rotate a 3D volume projection, use the last 128 colors
; INPUT:
;       VOL       - 3D input data array
; OUTPUT:
;       IMG       - 2D projected image obtained 
; KEYWORD:
;	RXYZ      - ratation angle about X,Y,Z axes, default [30,30,0] degree
; 	DEPTH_Q   - Depth queuing [0.-1.]: default 0.5
; 	OPQ       - Opacity of each cell [0.-1.]: default 0
;	CONTRAST  - Specify the maximum color contrast used if set
;	WIDTH     - window width
;	HEIGHT    - window height
;	HIGHTLIGHT- highlight data color 
;	TRANSFORM - Specify the initial transform matrix 
;	SHOWLOAD  - Specify the showload for animation
;

	if xregistered("XInterAnimage") then begin
	tmp = dialog_message(['Error in XINTERANIMATE:','', $
	'Only one animation at a time is allowed.'],/error)
		return
	end
if n_elements(vol) eq 0 then begin
; First, create some data:
vol = RANDOMU(S, 40, 40, 40) 
 opaque = RANDOMU(S, 40, 40, 40) 
 FOR I=0, 10 DO opaque = SMOOTH(opaque, 3) 
 opaque = BYTSCL(opaque(3:37, 3:37, 3:37), TOP=25B)
end

s = size(vol)
; Set up the view:
xmin = 0 & ymin = 0 & zmin = 0
xmax = s(1) & ymax = s(2) & zmax = s(3) 
dim_max = max([xmax,ymax,zmax])
ratio = [xmax,ymax,zmax]/float(dim_max)     ; espect ratio

if dim_max gt 160 then vol=congrid(vol,ratio(0)*160,ratio(1)*160,ratio(2)*160)
s = size(vol)
if s(0) ne 3 then return
	if keyword_set(contrast) then begin
	v1 = min(vol)
	v2 = max(vol)
	vol = byte((float(vol)-v1)/(v2-v1)*(!d.table_size-1))
	end
; use last 128 colors only
if keyword_set(highlight) then begin
	v_min = min(vol)
	v_max = max(vol)
	vol = 127 + 128*(float(vol)-v_min)/(v_max-v_min)
end

opaque = 0*vol
if keyword_set(Opq) then begin
	if Opq gt 1 then Opq=1
	opaque=opaque + Opq*vol
end

depth =0.5
if n_elements(depth_q) then depth = depth_q

Rotation=[30,30,0]
if n_elements(RXYZ) then Rotation=Rxyz

 XSIZE=400
 YSIZE=400
if n_elements(WIDTH) then xsize=width
if n_elements(HEIGHT) then ysize=height

xmax = s(1) & ymax = s(2) & zmax = s(3) 
!X.S = [-xmin, 1.0] / (xmax - xmin)
!Y.S = [-ymin, 1.0] / (ymax - ymin)
!Z.S = [-zmin, 1.0] / (zmax - zmin)
;if keyword_set(transform) then print,transform
scale = .5 * ratio 
trans0=[.5,.5,.5]
T3D, /RESET
T3D, TRANSLATE=-trans0, SCALE=scale, ROTATE=rotation
T3D, TRANSLATE=trans0

; Generate and display the image:
	img = PROJECT_VOL(vol, s(1),s(2),s(3), DEPTH_Q=depth, $
	  OPAQUE=opaque, TRANS=(!P.T)) 
	if keyword_set(showload) then tv,img
	if keyword_set(contrast) then begin
	v1 = min(img)
	v2 = max(img)
	img = ceil((float(img)-v1)/(v2-v1)*(!d.table_size-1))
	end

axis,0,0,0,/noerase, color=255,/T3D, $
/xaxis,xrange=[0,s(0)],xtitle='X',xcharsize=4,xthick=2, $
/yaxis,yrange=[0,s(1)],ytitle='Y',ycharsize=4,ythick=2, $
/zaxis,zrange=[0,s(2)],ztitle='Z',zcharsize=4,zthick=2
	sz = size(img)

	if keyword_set(Rotate) then begin
	nframes =30 
	temp_array=make_array(xsize,ysize,nframes/2+1)
	for i=0, nframes -1 do begin
	T3D, TR=[-.5,-.5,-.5], ROTATE= [5,5,0] ; i*[0,2,0]
	T3D, TRANSLATE=[0.5, 0.5, 0.5]
	img = PROJECT_VOL(vol, s(1),s(2),s(3), DEPTH_Q=depth, $
	  OPAQUE=opaque, TRANS=(!P.T)) 
if i mod 2 eq 1 then begin
	if keyword_set(showload) then tvscl,img
	if keyword_set(contrast) then begin
	v1 = min(img)
	v2 = max(img)
	img = ceil( (!d.table_size-1)*(float(img)-v1)/(v2-v1))
	end
	temp_array(*,*,i/2) = img(*,*)
end
	endfor
	volviewer_showmovie,temp_array,showload=showload ;image_array

	end
END

;------------------------------------------------------------
PRO volviewer_historyproject,vol_array,rotation=rotation,transform=transform, $
smooth=smooth,coarse=coarse,scale=scale,contrast=contrast,highlight=highlight, $
depth=depth,opq=opq,showload=showload
; this routine animate the vol_array history with user specified 
;         transform matrix and rotation ( fixed viewing position )
; INPUT:  vol_array(W,H,D,nframes)  - byte array
; KEYWORD:
;        TRANSFORM  - specifies the 3D projection transformation matrix 
;        ROTATION   - specifies additional rotation angles [Rx,Ry,Rz]
;        SMOOTH - if specified, the data smoothing is desired
;	CONTRAST
;	SHOWLOAD
;	OPQ
;	DEPTH
;

; First, create some data:
if n_elements(vol_array) eq 0 then begin
vol_array = RANDOMU(0, 40, 40, 40, 5) 
end

s = size(vol_array)
if s(0) ne 4 then return
dim_max = max([s(1),s(2),s(3)])
ratio = s(1:3)/float(dim_max)   	; aspect ratio
if dim_max gt 160 then coarse = 1
if keyword_set(coarse) then begin
	n_s = 160 * ratio
	temp_array = make_array(n_s(0),n_s(1),n_s(2),s(4))
	for i=0,s(4)-1 do begin	
	temp = vol_array(*,*,*,i)
	temp = congrid(temp,n_s(0),n_s(1),n_s(2))
	temp_array(*,*,*,i) = temp(*,*,*)
	end
vol_array = temp_array
end

v_max = max(vol_array)
v_min = min(vol_array)

; rotation=[ax,ay,az]
  rotate = [30,30,0]
  if keyword_set(rotation) then rotate=rotation

; Set up the view:
xmin = 0 & ymin = 0 & zmin = 0
xmax = s(1) & ymax = s(2) & zmax = s(3) 

!X.S = [-xmin, 1.] / (xmax - xmin)
!Y.S = [-ymin, 1.] / (ymax - ymin)
!Z.S = [-zmin, 1.] / (zmax - zmin)

tscale = 0.5 * ratio ;  [.5,.5,.5]
if keyword_set(scale) then tscale = scale
trans0 = [.5,.5,.5]
T3D,/RESET
;if keyword_set(transform) then !P.T = transform 
;T3D,translate=trans0,SCALE=tscale
 T3D, TRANS= -trans0     ; move center to origin
 T3D, SCALE=tscale , ROTATE= rotate 
 T3D, TRANSLATE=trans0           ; restore origin
 
nframes = s(4)
if keyword_set(showload) then WINDOW, 0, XSIZE=400, YSIZE=400 else $
WINDOW, 0, XSIZE=400, YSIZE=400,/pixmap
image_array = make_array(400,400,nframes)

 s = size(vol_array)
 ;opaque = RANDOMU(0, s(1),s(2),s(3)) 
 ;FOR I=0, 10 DO opaque = SMOOTH(opaque, 3) 

depthq = 0.5
if keyword_set(depth) then depthq = depth

	if keyword_set(highlight) then $ 
	vol_array = 127 + ceil(128 * (float(vol_array )-v_min)/(v_max-v_min))

FOR J=0,nframes-1 DO BEGIN
	vol = vol_array(*,*,*,J)
	opaque = 0*vol 
	if keyword_set(opq) then opaque = opaque+ opq * vol 

	if keyword_set(smooth) then  FOR I=0, 10 DO vol = SMOOTH(vol, 5) 

; Generate and display the image:
	img = PROJECT_VOL(vol, s(1),s(2),s(3), DEPTH_Q=depthq, $
	  OPAQUE=opaque, TRANS=(!P.T)) 
	if keyword_set(showload) then tv,img
	image_array(*,*,j) = img(*,*)
ENDFOR
	wdelete,0
	v1 = min(image_array)
	v2 = max(image_array)

	if keyword_set(contrast) then $ 
		image_array = (!d.table_size-1)*(image_array-v1)/(v2-v1)
	volume_animator,image_array,title='history_project'
END

;------------------------------------------------------------
PRO volviewer_openhistorydata,vol_array
; read in history of 3D volume data
; RETURN:  vol_array(d1,d2,d3,nstep)
; Use the file selection dialog to pick file name *history*.xdr  filter
;
;   should contains the following xdr arrays 
;   dim[4]         - [d1,d2,d3,nstep]
;   ctab[256,3]    - color tables
;   vol[d1,d2,d3]  - nstep sets of raw vol data
; 
;
	file = dialog_pickfile(/read,get_path=path,/must_exist, $
		filter='*history*.xdr', $
		title='Read 3D History XDR Array')
	if file eq '' then return
	error=0
	if error eq 0 then begin
	xdr_open,unit,file,error=error
	xdr_read,unit,dim
	if n_elements(dim) ne 4 then begin
		ret=dialog_message('It is not a 3D volume history file',/error)
		return
	end
	xdr_read,unit,ctab
	vol_array=make_array(dim(0),dim(1),dim(2),dim(3))
	for i=0,dim(3)-1 do begin
	 xdr_read,unit,vol
	 vol_array(*,*,*,i) = vol(*,*,*)
	end
	end
	if error ne 0 then begin
		r=dialog_message(!error_state.msg,/Error)
		return
	end
	if n_elements(unit) then xdr_close,unit

END

;------------------------------------------------------------
PRO volviewer_dataToColorByte,vol,view=view
;+
; NAME:
;	VOLVIEWER_DATATOCOLORBYTE
;
; PURPOSE:
; 	This routine converts the input data array vol to an equivalent 
;  	color index byte array. It linearly scales the values between 
;	min and max to 0 and 255. The original array been changed to 
;	color byte array.
;
; CALLING SEQUENCE:
;      volviewer_dataToColorByte, Vol, View=view 
;
; INPUT:
; 	VOL  - Input variable can be any 2D or 3D data array 
; 
; OUTPUT:
; 	VOL  - Returns a color byte array converted from the input array
;
; KEYWORD: 
;   VIEW - if specified, if the input is a 2D data array it
;          will be displayed as 2D image
; EXAMPLE:
;	v = dist(100)
;	volviewer_dataToColorByte,v,/view 
;-
	y = dist(100)*100
	if n_elements(vol) eq 0 then vol = y
	ymax = max(vol)
	ymin = min(vol)
	ncolor = !d.table_size-1   ; !d.n_colors
	vol = (vol-ymin)*ncolor/(ymax-ymin)
	vol = byte(vol)

	if keyword_set(view) then begin
	sz=size(vol)
	if sz(0) eq 2 then begin
	erase,color=ncolor
	tv,vol
	end
	if sz(0) eq 3 then volviewer_project,vol
	end
END


;------------------------------------------------------------
PRO volviewer_readRawImages,vol,file=file,delete=delete
; NAME:
;	VOLVIEWER_READRAWIMAGES
;
; PURPOSE:
;	This routine reads a sequence of  images data[3,width,height]
;	from the default  'vol_images.xdr' file and converts to
;	2D image , displays it, and  populates the  volume array
;	vol[width,height,nframe]
; 
; OUTPUT:
;  Vol[width,height,nframe] - return the vol array ready for movie display
;      which is ready for volviewer_showmovie routine
; KEYWORD: 
;  File         - speicifies the input file name, default is
;                 vol_images.xdr 
;        File Contents
;        dim[3]       - object window images dimensions [width,height,nframe]
;        ctab[256,3]  - associated color table
;        im[3,width,height]  - true color image array for each frame
;  Delete       - if specified, the window 0 will be deleted at the end of read
;
	filename = 'vol_images.xdr'
	if keyword_set(file) then filename = file
	xdr_open,unit,filename
	xdr_read,unit,dim
	vol = make_array(dim(0),dim(1),dim(2),/byte)

	xdr_read,unit,ctab
	tvlct,ctab(*,0),ctab(*,1),ctab(*,2)

	 window,0,xsize=dim(0),ysize=dim(1)

	for i=0,dim(2)-1 do begin
	 xdr_read,unit,im   ; im[3,width,height]
	 if !d.n_colors le 256 then begin
		sim = color_quan(im,1,R,G,B)  ; ,colors=128)
		tvlct,r,g,b
		tv,sim
	 endif else begin
	 device,decompose=1
	 tv,im,/true
	 device,decompose=0
	 end
 	 im = tvrd()
	 vol(*,*,i) = im(*,*)
	end
	xdr_close,unit

	if keyword_set(delete) then wdelete,0
	tvlct,ctab(*,0),ctab(*,1),ctab(*,2)
END

;------------------------------------------------------------
PRO volviewer_3Dmovie,vol_array,save=savE,HIGHLIGHT=HIGHLIGHT,CONTRAST=CONTRAST, $
opaque=opaque,depthq=depthq,width=width,height=height,showload=showload,transform=transform
;+
; NAME:
;	VOLVIEWER_3DMOVIE
;
; PURPOSE:
;	This routine accepts a 4D volume history array as input and 
;	construct the projected 2D image history and pass the 2D image array
;	to the xinteranimate program. 
;
; CALLING SEQUENCE:
;	Volviewer_3Dmovie, vol_array, /showload,/contrast
;
; INPUT : 
;   VOL_ARRAY  - Input history array of volume data
;                vol_array [W,H,D,NSTEP]
;                where   W     - X dimension
;                        H     - Y dimension
;                        D     - Z dimension
;                        NSTEP - time steps 
;
; KEYWORD:
;   HIGHLIGHT  - if specified, use the last 128 colors from the color table
;   CONTRAST   - if specified, use most color contrast for each frame
;   OPAQUE     - specified opaque level [0.-1.]
;   DEPTHQ     - specified depth level [0.-1.]
;   WIDTH      - specified window xsize, default 400
;   HEIGHT     - specified window ysize, default 400
;   SHOWLOAD   - show TV load if specified
;   SAVE       - save the projected 2D image array to 'movie.xdr'
;
; EXAMPLE:
;	volviewer_3Dmovie,vol_array
;-

sz = size(vol_array)
if sz(0) ne 4 then begin
	rt = dialog_message(/error,'Vol_array[w,h,d,nstep] must be provided')
	return
end

xsize=400 & ysize=400
if keyword_set(width) then xsize=width
if keyword_set(height) then ysize=height
if keyword_set(showload) then window,0,xsize=xsize,ysize=ysize else $
window,0,xsize=xsize,ysize=ysize,/pixmap

rim = make_array(xsize,ysize,sz(4))

; write header info to vol_images.xdr file

	nstep = sz(4)
        for i=0,nstep-1 do begin
	y = vol_array(*,*,*,i)
        volviewer_project,y,im,opq=opaque,depth_q=depthq,highlight=highlight, $
                contrast=0, showload=showload,transform=transform

        rim(*,*,i) = im(*,*)
	if keyword_set(showload) then tv,IM
        end

        if keyword_set(save) then  volviewer_savemovie,rim
        wdelete,0
        volviewer_showmovie, rim, showload=showload

END

;------------------------------------------------------------
PRO volviewer_savemovie, vol 
;  this routine writes the 3D data array vol to 'movie.xdr' file
;  INPUT:  vol(width,height,nframe)
;  OUTPUT:  'movie.xdr' file is created/updated
;
	xdr_open,unit,'movie.xdr',/write
	xdr_write,unit,vol
	xdr_close,unit
END
;------------------------------------------------------------
PRO volviewer_readmovie, vol 
;  this routine reads the 'movie.xdr' file and returns nframe of 2D image
;  BYTE data array which is expressed in the color table numbers
;  OUTPUT:  vol(width,height,nframe)
;
	xdr_open,unit,'movie.xdr'
	xdr_read,unit,vol
	xdr_close,unit
END
;------------------------------------------------------------
PRO volviewer_showmovie, vol,showload=showload 
;+
; NAME:
;	VOLVIEWER_SHOWMOVIE
;
; PURPOSE:
;	This routine will show the image array frame by frame from the 3D
;	BYTE array where color table is already factored into the 
;	image data array.  It calls the xinteranimate to display the movie.
;
; CALLING SEQUENCE:
;	volviewer_showmovie, Vol,/showload
;
; INPUT: 
;    VOL	-  Image array vol(width,height,nframe)
;		where  width   - image array xsize
;		       height  - image array ysize
;		       nframe  - number of image frame in array
; KEYWORDS:
;    SHOWLOAD  - if specified, display the image while it is loading
;
; EXAMPLE:
;	volviewer_showmovie,VOL
;-
	sz = size(vol)
	if sz(0) ne 3 then begin
		print,'input data dimension error'
	end

	xinteranimate,set=[sz(1),sz(2),sz(3)],showload=showload
	for i=0,sz(3)-1 do xinteranimate,frame=i,image=vol(*,*,i)
	xinteranimate,10
END
;------------------------------------------------------------
PRO volume_animator,H,axis=axis,factor=factor,title=title
;+
;  VOLUME_ANIMATOR
;
; The volviewer_animator procedure allows the user to use the IDL xinteranimate
; to view the 2D images of a 3D data array sequentially alang any direction.
; The default dimension is slice along the 3rd dimension. 
; Through controlling the frames/sec slider bar, a user can easily flip
; through the 2D image in any direction.
;
; SYNTAX
;    VOLUME_ANIMATOR, H [,Axis=3] [,Factor=2] [,TITLE='Any text']
;
; ARGUMENTS:
;	H[N,M,K] - input 3D data array, the value of each dimensions from 
;                  the data array must be greater than 3 
;
; KEYWORDS:
;	AXIS     - default to 3, to specify the slicing axis for animation
;                  It can be 1 or 2 or 3.
;       FACTOR   - image blow up factor for animation, default is 2
;                  if the raw image size exceeds 250, it will be reset
;                  to 2
;       TITLE    - string to specify the XINTERANIMATE window title bar
;                  if not specified, default 'XinterAnimate' will be used
; EXAMPLE:
; Read the IDL cloud data and slice along the second axis, and blowup
; the image by a factor of 4
;
;	.run volviewer
;	volviewer_readCloud,vol
;	volume_animator,vol,axis=2,factor=4
;-

; delete the current xinteranimate if exists
COMMON XInterAnimate_com,topbase,animatebase,pwin,mpegID
        if n_elements(topbase) then widget_control,topbase,/destroy,bad_id=bad

	; Read the images into variable H

  if n_elements(H) eq 0 then begin
    OPENR, unit, FILEPATH('abnorm.dat', SUBDIR=['examples','data']), $
     /GET_LUN
    H = BYTARR(64, 64, 16) 
    READU, unit, H 
    CLOSE, unit 
  end
  
	sz = size(H)
	if sz(0) lt 3 then begin
		r=dialog_message('Wrong type of data detected',/error)
		return
	end

	ff = 2
	if keyword_set(factor) then ff = factor
	if ff le 0 then ff=2
	k = 3  ; default z axis
	if keyword_set(axis) then begin 
		if axis lt 0 or axis gt 3 then return
		k = axis
	end
	if k eq 3 and sz(1) gt 250 or sz(2) gt 250 then ff=1
	if k eq 2 and sz(1) gt 250 or sz(3) gt 250 then ff=1
	if k eq 1 and sz(2) gt 250 or sz(3) gt 250 then ff=1
	if k eq 3 then begin 
	  nH  = rebin(H,ff*sz(1),ff*sz(2),sz(3))
	end
	if k eq 2 then begin
	  nH  = bytarr(sz(1),sz(3),sz(2))
	  FOR I=0,sz(k)-1 DO nH(*,*,I) = H[*,I,*]
	  nH  = rebin(nH,ff*sz(1),ff*sz(3),sz(2))
	end
	if k eq 1 then begin
	  nH  = bytarr(sz(2),sz(3),sz(1))
	  FOR I=0,sz(k)-1 DO nH(*,*,I) = H[I,*,*]
	  nH  = rebin(nH,ff*sz(2),ff*sz(3),sz(1))
	end
	  sz = size(nH)
	if keyword_set(title) eq 0 then begin
	  title = 'XinterAnimate - Z frames'
	  if k eq 1 then title = 'XinterAnimate - X frames'
	  if k eq 2 then title = 'XinterAnimate - Y frames'
	end

	; Load the images into XINTERANIMATE:
	  XINTERANIMATE, SET=[sz(1), sz(2), sz(3)], /SHOWLOAD,title=title

	  ; Initialize XINTERANIMATE:
	  FOR I=0,sz(3)-1 DO XINTERANIMATE, FRAME = I, IMAGE = nH[*,*,I]

	; Play the animation:
	XINTERANIMATE, 10, /KEEP_PIXMAPS

END

