; 
; IDL Event Callback Procedures
; volviewer_eventcb
; 
; Generated on:	10/25/2001 17:49.19
; 
@PS_open.pro
@xdr_open.pro
@volume_animator.pro

PRO volviewer_rawdata2Images,vol_array,sEvent
;  For truecolor visual devices, e.g 'WIN'
;  use the volume object to construct the image history array
; INPUT:
; vol_array[d1,d2,d3,nstep]  - history of volume array
; OUTPUT:  
;  -display the vol object with new vol data
;  -get the image array from the view display
;  -save the image array to the temporary file
;   use temp file 'vol_data2Images.xdr' to store the raw images from the
;   read TV image object
;  -call volviewer_readRawImages to get image frames
;  -call volviewer_showmovie
;
       WIDGET_CONTROL, sEvent.top, GET_UVALUE=sState, /NO_COPY
	sz = size(vol_array)
	width = sState.drawsize(0)
	height = sState.drawsize(0)
	nstep = sz(4)
	vol = make_array(sz(1),sz(2),sz(3))
	oWindow=sState.owindow
	oVol = sState.oVol
	oVol->getProperty,data0=oldvoldata
	oVol->getProperty,RGB_TABLE0=ctab

	xdr_open,unit,'vol_data2Images.xdr',/write
	xdr_write,unit,[width,height,nstep]
	xdr_write,unit,ctab

	for i=0,nstep-1 do begin
	  vol(*,*,*) = vol_array(*,*,*,i)
	  oVol->setProperty,data0=vol
	  oWindow->draw,sState.oview
	  oImg = sState.owindow->read()
	  oImg->getProperty,data=im
	  xdr_write,unit,im
	end
	xdr_close,unit
	oVol->setProperty,data0=oldvoldata
       WIDGET_CONTROL, sEvent.top, SET_UVALUE=sState, /NO_COPY

	volviewer_readRawImages,ims,file='vol_data2Images.xdr',/delete
	volviewer_showmovie,ims
END

PRO volviewersave_vectrack,event
; this routine extract the volume data from the ovol object
; then save the data in d_vectrack data structure format in 'vectrack.dat' 
; then bring up the d_vectrack program
;
	widget_control,Event.top,get_uvalue=state,/no_copy
;	state.ovol->getProperty,data0=vol
	vol = state.data
	widget_control,Event.top,set_uvalue=state,/no_copy
	save_vectrack,vol
	d_vectrack,LOADDATA='vectrack.dat' ;,GROUP=Event.top
END

PRO save_vectrack,vol
; this routine convert any time snapshot of a 3D data and 
; construct the data in the data structure ready to feed into the
; d_vectrack program in 'vectrack.dat'
; 
;   Original definition in d_vectrack
;   U,V,W : 3D velocity array
;   pb,tb,mb  : pressure,temperature,density 3D array
;   For testing purpose, we set every variable same as the input vol 
; 
help,vol
	vec_track_data = { $
		fU:ptr_new(/allocate_heap), $
		fV:ptr_new(/allocate_heap), $
		fW:ptr_new(/allocate_heap), $
		bP:ptr_new(/allocate_heap), $
		bT:ptr_new(/allocate_heap), $
		bM:ptr_new(/allocate_heap) $
	}
	*vec_track_data.fU = vol
	*vec_track_data.fV = vol
	*vec_track_data.fW = vol
	*vec_track_data.bP = vol
	*vec_track_data.bT = vol
	*vec_track_data.bM = vol
	
	save,filename='vectrack.dat',vec_track_data
END

PRO restore_vectrack,u,v,w,pb,mb,tb
; this routine restore the 'vectrack.dat' and ready running
; d_vectrack program
	restore,'vectrack.dat'
help,vec_track_data
	u= *(vec_track_data.fu[0])
	v= *(vec_track_data.fu[0])
	w= *(vec_track_data.fu[0])
	pb = *(vec_track_data.fu[0])
	mb = *(vec_track_data.fu[0])
	tb = *(vec_track_data.fu[0])
help,u,v,w,pb,mb,tb
END

;-----------------------------------------------------------------
PRO volviewer_openvolhistory, Event,readonly=readonly,fliponly=fliponly,$
project=project
; this routine read raw 3D history data array 
; *rawhistory*.xdr file contains history of vol_array only
;    vol_array[w,h,d,nframes]
; and reset the vol history pointer 
; KEYWORD:
;	READONLY  - read vol history data only
;	FLIPONLY  - flip through vol history data only, use xinteranimate
;	PROJECT   - use xinteranimate to view the window 2D projection
;
;

    IF KEYWORD_SET(fliponly) THEN BEGIN
       WIDGET_CONTROL, Event.top, GET_UVALUE=vState, /NO_COPY
	catch,error_status
	if error_status ne 0 then begin
	r = dialog_message(/error,'You have to load in history data first!!')
	goto,nextline
	end
	if ptr_valid(vState.history) then vol_array = *vState.history 
nextline:
       WIDGET_CONTROL, Event.top, SET_UVALUE=vState, /NO_COPY
	if n_elements(vol_array) eq 0 then return
	sz=size(vol_array)
    ENDIF ELSE BEGIN
       WIDGET_CONTROL, Event.top, GET_UVALUE=vState, /NO_COPY
	cwd = vState.path
	if cwd eq '' then CD,current=cwd
       WIDGET_CONTROL, Event.top, SET_UVALUE=vState, /NO_COPY
	file = dialog_pickfile(/read,path=cwd,get_path=path,/must_exist, $
		filter='*rawhistory*', $
		title='Read 3D XDR raw history Array')
	if file eq '' then return

widget_control,/hourglass

	error=0
	if error eq 0 then begin
	xdr_open,unit,file,error=error
	xdr_read,unit,vol_array
	end
	if error ne 0 then begin
		r=dialog_message(!error_state.msg,/Error)
		return
	end
	if n_elements(unit) then xdr_close,unit
	sz=size(vol_array)
	if sz(0) ne 4 then begin
		help,vol_array
		d = dialog_message('This is not a raw vol_array[w,h,d,n] data',/error)
		return
	end
	END

       WIDGET_CONTROL, Event.top, GET_UVALUE=sState, /NO_COPY
	*sState.history = vol_array
	contrast = sState.contrast 
	showload = sState.showload 
	highlight = sState.highlight 
	depthq = sState.depthq 
	opaque = sState.opaque 
	drawsize = sState.drawsize 
	skips = sState.skips 
	last = sState.last 
	sState.last = sz(4)
;	sState.onoff = 1
	sState.omodelTranslate->getproperty,Transform=transform
	hslicerWID = sState.hslicerWID
       WIDGET_CONTROL, Event.top, SET_UVALUE=sState, /NO_COPY

	vol = vol_array(*,*,*,sz(4)-1)
	volviewer_open,Event,vol=vol

	if XRegistered('volviewer_historyslicer') then $
	widget_control,hslicerWID,/destroy

	if keyword_set(readonly) then return

	; extract  every other skips
	nstep = sz(4)
	dd = 2
	if skips ne dd then dd = skips
	if dd gt nstep then dd = nstep
	num = nstep/dd + 1
	temp_array = make_array(sz(1),sz(2),sz(3),num)
	ip=0
	for i=0,num-1 do begin
	ip = dd*i
	if ip ge nstep then ip = nstep-1
	temp_array(*,*,*,i) = vol_array(*,*,*,ip)
	end

if keyword_set(project) then begin
	volviewer_3Dmovie,temp_array,width=drawsize(0),height=drawsize(1), $
	   showload=showload,transform=transform, $
	   depthq=depthq,opaque=opaque,highlight=highlight,contrast=contrast

endif else begin
	; try use object to plot history

   loops:
	for i=0,num-1 do begin
	vol = temp_array(*,*,*,i)
	widget_control,Event.top,get_uvalue=state,/no_copy
;	onoff = state.onoff
	state.omodelTranslate->setProperty,TRANSFORM=transform
	state.ovol->setproperty,data0=vol
	state.owindow-> draw, state.oview
	widget_control,Event.top,set_uvalue=state,/no_copy
	end
;	if onoff eq 1 then goto, loops
end
END
;-----------------------------------------------------------------
pro volviewer_open, Event, vol=vol
; this routine use file selection box to pick any 3D vol*.xdr file
; each file should contain only one single 3D data array saved by
; xdr_write  
;
	widget_control,/hourglass
	widget_control,Event.top,get_uvalue=state,/no_copy

if keyword_set(vol) eq 0 then begin
	cd,current=cwd
	if state.path ne '' then  cwd=state.path
	file = dialog_pickfile(/read,path=cwd,get_path=path,/must_exist, $
		filter='vol*.xdr', $
		title='Read 3D XDR Array')
	if file eq '' or file eq cwd then begin
		widget_control,Event.top,set_uvalue=state,/no_copy
		return
	end

	error=0
	if error eq 0 then begin
	xdr_open,unit,file,error=error
	xdr_read,unit,vol
	end
	if error ne 0 then begin
		r=dialog_message(!error_state.msg,/Error)
		widget_control,Event.top,set_uvalue=state,/no_copy
		return
	end
	if n_elements(unit) then xdr_close,unit
	sz=size(vol)
	if sz(0) ne 3 then begin
		help,vol
		d = dialog_message('This is not a volume data',/error)
		widget_control,Event.top,set_uvalue=state,/no_copy
		return
	end

	state.path = path
	state.file = file
end
	volviewerUpdateObjects,state,vol
	widget_control,Event.top,set_uvalue=state,/no_copy
	
end

PRO volviewer_readvol,file,error,vol
        error=0
        if error eq 0 then begin
        xdr_open,unit,file,error=error
        xdr_read,unit,vol
        end
        if error ne 0 then begin
                r=dialog_message(!error_state.msg,/Error)
                return
        end
        if n_elements(unit) then xdr_close,unit
        sz=size(vol)
        if sz(0) ne 3 then begin
                help,vol
                d = dialog_message('This is not a volume data',/error)
                return
        end

end
;-----------------------------------------------------------------
pro volviewer_save, Event
; extract the 3D data from the volume object and saved in an xdr file
;
	widget_control,Event.top,get_uvalue=volviewer_state,/no_copy
	volviewer_state.ovol->getProperty,data0=vol
	widget_control,Event.top,set_uvalue=volviewer_state,/no_copy
	file = dialog_pickfile(/write,get_path=path,filter='vol*.xdr', $
		title='Write 3D XDR Array')
	if file eq '' then return
	error=0
	if error eq 0 then begin
	xdr_open,unit,file,/write,error=error
	xdr_write,unit,vol,error=error
	end
	if error ne 0 then begin
	 r=dialog_message(!error_state.msg,/Error)
	 return
	end
	if n_elements(unit) then xdr_close,unit
end
;-----------------------------------------------------------------
pro volviewer_savevolhistory, Event
; save the 3D history data in an xdr file
;
	widget_control,Event.top,get_uvalue=volviewer_state,/no_copy
	catch,error_status
	if error_status ne 0 then goto,nextline
	if ptr_valid(volviewer_state.history) then $
	vol_array = *volviewer_state.history
nextline:
	widget_control,Event.top,set_uvalue=volviewer_state,/no_copy
	if n_elements(vol_array) eq 0 then return
	sz = size(vol_array)
	if sz(0) ne 4 then return

	file = dialog_pickfile(/write,get_path=path,filter='*rawhistory*.xdr', $
		title='Write 3D History XDR Array')
	if file eq '' then return
	error=0
	if error eq 0 then begin
	xdr_open,unit,file,/write,error=error
	xdr_write,unit,vol_array,error=error
	end
	if error ne 0 then begin
	 r=dialog_message(!error_state.msg,/Error)
	 return
	end
	if n_elements(unit) then xdr_close,unit
end
;-----------------------------------------------------------------
pro volviewer_printer, Event
	PS_printer
end

;-----------------------------------------------------------------
PRO volviewer_print, Event,arr=arr,ctable=ctable
	widget_control,Event.top,get_uvalue=volviewer_state,/no_copy
	oWindow = volviewer_state.owindow
	oIm = oWindow->read()
	oIm->getProperty,data=arr
	widget_control,Event.top,set_uvalue=volviewer_state,/no_copy
	sz = size(arr)
	xs = sz(2)
	ys = sz(3)
	if keyword_set(ctable) then begin
	arr = color_quan(arr,1,r,g,b) 
	sz = size(arr)
	xs = sz(1)
	ys = sz(2)
	end
	width = float(xs)/40
	height = float(ys)/40
	set_plot,'PS'
	device,filename='idl.ps',/color,bits=8, $
		/Courier,/Bold, scale_factor=1.1, $
		xsize=width,ysize=height
	if sz(0) eq 2 then begin
	tvlct,r,g,b
	tv,arr
	endif else TV,arr,/true
	PS_close
	PS_print, 'idl.ps'
END

pro volviewer_readconfig,path,file
	path=''
	file=''
	catch,status_error
	if status_error ne 0 then begin
		close,1
		return
	end
	openr,1,'volviewer.config'
	readf,1,path
;	readf,1,file
	close,1
end
pro volviewer_writeconfig,state
	if state.path ne '' and state.file ne '' then begin
	openw,1,'volviewer.config'
	printf,1,state.path
;	printf,1,state.file
	close,1
	end
end
;-----------------------------------------------------------------
pro volviewer_quit, Event
  widget_control,Event.Top,get_uvalue=state,/no_copy
  volviewer_writeconfig,state
  widget_control,Event.Top,set_uvalue=state,/no_copy
  widget_control,event.top,/destroy
end
;-----------------------------------------------------------------
pro volviewer_getcolortable,ctab
COMMON colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr

  ; create color palette
	reds = r_curr
	greens = g_curr
	blues = b_curr
  ctab = bytarr(!d.table_size,3)
  ctab = ctab+byte(!d.table_size-1)
  
  for i=0,!d.table_size-1 do begin
  ctab(i,0) = reds(i)
  ctab(i,1) = greens(i)
  ctab(i,2) = blues(i)
  end
end
pro volviewer_color, Event
	xloadct,group=Event.top,/modal
	WIDGET_CONTROL,Event.top,GET_UVALUE=state,/no_copy
	volviewer_getcolortable,ctab
	state.oVol->setProperty,RGB_TABLE0=ctab
  	state.owindow -> draw, state.oview
	WIDGET_CONTROL,Event.top,SET_UVALUE=state,/no_copy
end
;-----------------------------------------------------------------
pro volviewer_help, Event
str = ['', $
	'                       Introduction', $
	'                      -----------------------','', $
	'This program provides the IDL user with flexible ways of viewing 3D', $
	'data array. ','', $
	'It accept the 3D array on the command line or through loading XDR file', $
	'into the program.', '', $
	'	Usage:  volviewer, data','',$
	'	where data is a 3D data array.', '', $
 	'After the 3D data is loaded into the VolViewer, the user can view', $
	'the 3D volume data through 3D rotating controls or view the 2D data', $
	'through 2D crossectional animate tools. The XinterAnimation tool is', $
	'used in either 2D or 3D animation.','', $
	'Examine the volume by rotating the 3D object through mouse ', $
	'trackball or by constant angle rotating about the  coordinate', $
	'axes X,Y,Z with user controllable rotating angle, or 3D IsoSurface', $
	'animation through controling the rotation angles AX, AZ and contour level.', $
	'Examine the 2D crossection through 2D slicer or axial crosstional', $
	'animation.', $
	'',$
	'                      Volume Data File ',$
	'                      -----------------------','', $
	'The VolViewer uses the xdr_read, and xdr_write to load/save the ', $
	'volume data to disk file. The assumed file name starts with "vol"', $
	'prefix and end with ".xdr" suffix. For example to save a 3D data array',$
	'in the "vol_example1.xdr" in format accepted by the VolViewer','', $
	"       xdr_open,unit,'vol_example1.xdr',/write",$
	"       xdr_write,unit,data", $
	"       xdr_close,unit", $
	'', ' where data(width,height,depth) is a 3D data array','',$
	'                     3D History File', $
	'                      -----------------------','', $
	"The 3D history filename should contain the 'history' sub-string. ", $
	'Two types of history file supported: raw history file include a ',$
	'single 4D data array or a file consists of dimensions, color table,',$
	'plus a set of sequential volume data arrays.','', $
	'The 3D history file should contain the following information which',$
	'should be created by the xdr_open routines', $
	'','Type A File:', $
	"	xdr_open,unit,'rawhistory1.xdr',/write", $
	"	xdr_write,unit,vol_array", $
	"	xdr_close,unit", $
	'where volume array is a 4 dimension variable', $
	'      vol_array(width,height,depth,nsteps)','', $
	'Type B File:', $
	"       xdr_open,unit,'vol_history1.xdr',/wirte",$
	"       xdr_wrtie,unit,dim",$
	"       xdr_write,unit,ctab",$
	"       xdr_write,unit,vol(*,*,*,0)", $
	"       xdr_write,unit,vol(*,*,*,1)", $
	"       xdr_write,unit,vol(*,*,*,2)", $
	"           .....", $
	" where dim[4] -  [width,height,depth,nsteps]", $
	"       ctab[236,3] - color tables", $
	"       vol[width,height,depth,nsteps] - nsteps sets of volume data", "",$

	'                       User Interface ', $
	'                      -----------------------','', $
	'File->Read Volume...   - load volume data saved by xdr_read', $
	'      Write Volume...  - write the current volume data by xdr_write', $
	'      Printer...       - override the default PS printer setting', $
	'      Print            - send drawing area to a PS printer', $
	'      Quit             - quit the 3D volviewer', $
	'',$
	'Setup->Colors...    - use xloadct to change color table', $
	'       Setup...        - direct graphics variables for 3D->2D image plot',$
	'			  (hilight,contrast,opaque,depth_q,skip/frames', $
	'       Aspect Ratio - On, Equal Aspect Ration used for voluem plot (default)', $
	'                      Off, Normalized cube used for volume plot', $
	'View(2D)->Z Axis Animator... - animate Z slices', $
	'       Y Axis Animator... - animate Y slices', $
	'       X Axis Animator... - animate X slices', $
	'       Animator Factor X2/X4... - toggle drawing area of animation program', $
	'',$
	'View(3D)->Axis On/Off  - toggle on/off the 3D axis', $
	'       Axis W/O Label  - draw axis with/without labels',$
;	'       Rotate...       - rotate the volume object with animation', $
	'       Test 3D Movie... - display the sample 3D movie ', $
	'       Open 3D History File... - load Type B 3D volume history file', $
	'                              and generate movie from the data', $
	'Tools->2D Slicer...     - call view3d_2d slicer', $
	'	D_vectrack...    - run modified version of d_vectrack', $
	'                          visualization program with current 3D data', $
	'',$
	'Vol_History(4D)->Read History Data(4D Array)... ',$
	'                       - load Type A 4D volume history file', $
	'             Flip Thru History Data (4D Array)...', $
	'                       - animate volume history data with skips', $
	'                         as defined by the View(3D)->Setup... dialog',$
	'             Write History Data (4D Array)...', $
	'                       - save history data as a raw 4D data array ',$
	'			  (Type A file)', $
	'',$
	'Help->About...     - show this help page', $
	'',$
	'3D Volume Rotation Control:', $
	'       Axis X,Y,Z  - Rotate about checked coordinate axis', $
	'       Degree      - Specify degrees per rotation', $
	'       Start       - Start rotation', $
	'       Stop        - Stop rotation', $
	'       Reset       - Reset to initial position', $
	'',$
	'3D IsoSurface Control:', $
	'	AX       - Specify the rotation angle about X axis',$
	'	AZ       - Specify the rotation angle about Z axis',$
	'	Level    - Specify the IsoSurface value',$
	'	Animate  - Run IsoSurface animator',$
	'', $
	'Drawing Area:', $
	'	Render the volume with 3 axis drawn', $
	'	TrackBall - rotate and position the volume', $
	'']
	xdisplayfile,text=str,title='VolViewer Help'
end
; 
; Empty stub procedure used for autoloading.
; 
pro volviewer_eventcb
end
